import os

import typer
from loguru import logger
from pyfzf.pyfzf import FzfPrompt

from jamexp.hyd.folder import extract_all_exp


def extract_dst_run(fname: str = ".runs"):
    rtn = []
    for sub_f in os.listdir(fname):
        if "j*_" in sub_f:
            sub_f = sub_f[3:]

        rtn.append(sub_f)
    return [item.replace("_", "/", 1) for item in rtn]


def pin_run(fname: str = "outputs", dst: str = ".runs"):
    dst_runs = extract_dst_run(dst)
    candidates = extract_all_exp(fname)
    unduplicate = []
    for item in candidates:
        for dst_item in dst_runs:
            if dst_item in item:
                break
        unduplicate.append(item)

    fzf = FzfPrompt()
    try:
        select = fzf.prompt(unduplicate, "--multi")
    except Exception as error:  # pylint: disable=broad-except
        print("FZF error: {}".format(error))

    for item in select:
        time_it = item.split("/")
        target = f"{time_it[-2]}_{time_it[-1]}"
        logger.info(f"{item} -> {dst}/{target}")
        os.symlink(item, f"{dst}/{target}")


def main():
    typer.run(pin_run)



import datetime
import os.path as osp
import shutil
import socket

import wandb
from loguru import logger

# https://docs.python.org/3/library/datetime.html#strftime-and-strptime-behavior
TIME_PARTTERN = "%Y-%m-%dT%H:%M:%S"


def extract_time(run):
    """extract run info

    :param run: [description]
    :type run: [type]
    :return: duration, run2now
    :rtype: datetime.timedelta
    """
    create_time_str = run.createdAt
    last_time_str = run.heartbeatAt
    start_time = datetime.datetime.strptime(create_time_str, TIME_PARTTERN)
    end_time = datetime.datetime.strptime(last_time_str, TIME_PARTTERN)
    now_time = datetime.datetime.now()
    duration = end_time - start_time
    run2now = now_time - end_time
    return duration, run2now


def delete_run(run, delete=True, is_all=False):
    """delete a wandb run on server

    :param run: wandb run object
    :type run: wandb.run
    :param delete: delete local run if exists and delete remote run, defaults to True
    :type delete: bool, optional
    :param is_all: delete runs not run on this machine, defaults to False
    :type is_all: bool, optional
    """
    # FIXME: should be a better approach to get hostname
    delete_info = f"{run.name}\t{run.url}\n"
    if not is_all and run.config["host"] != socket.gethostname():
        delete_info += f"\t{run.config['host']}! not no this machine!\n"
        logger.info(delete_info)
    else:
        cfg = run.config
        if "run_path" in cfg:
            if osp.exists(cfg["run_path"]):
                if delete:
                    shutil.rmtree(cfg["run_path"], ignore_errors=True)
                delete_info += f"\t Delete local run path: {cfg['run_path']}"
        logger.info(delete_info)
        if delete:
            run.delete()


def get_run_from_url(url: str):
    segs = url.split("/")
    entity = segs[-4]
    proj = segs[-3]
    run_id = segs[-1]
    wandb_id = "/".join([entity, proj, run_id])
    api = wandb.Api()
    try:
        run = api.run(wandb_id)
    except Exception as e:  # pylint: disable=broad-except
        run = None
        logger.critical(f"ERROR: {url}, \t{e}")
    return run


def delete_run_from_url(url: str):
    run = get_run_from_url(url)
    if run is not None:
        run.delete()



import os
import shutil

import typer
from loguru import logger
from pyfzf.pyfzf import FzfPrompt


def create_link(
    fname: str,
    is_delete: bool = typer.Option(False, "--d", help="delete dst if exists"),
    is_debug: bool = typer.Option(False, "--v", help="only show info, not create link"),
):
    """
    create softlink to another disk, mostly for data disk
    """
    cwd = os.getcwd()
    if cwd.startswith(f"/home/{os.getlogin()}"):
        path_split = cwd.split("/")
        path_relative_home = path_split[3:]
        dst = os.path.join(cwd, fname)
        if os.path.exists(dst):
            logger.warning(f"{dst} already exists")
            if is_delete:
                if os.path.islink(dst):
                    os.unlink(dst)
                else:
                    shutil.rmtree(dst, ignore_errors=True)
            else:
                exit(0)
        local_dist = [os.path.expanduser("~/dpdata"), os.path.expanduser("~/dpruns")]
        src_dist = [
            os.path.join("/", fdir, os.getlogin())
            for fdir in os.listdir("/")
            if "hd" in fdir or "hhd" in fdir or "ssd" in fdir or "scratch" in fdir
        ]
        src_dist.extend(local_dist)
        try:
            select = os.environ["DATA_DISK"]
        except Exception as error:  # pylint: disable=broad-except
            logger.warning("Env DATA_DISK not SET, use FZF")
            try:
                fzf = FzfPrompt()
                select = fzf.prompt(src_dist)
                logger.warning(f"src select disk: {select[0]}")
            except Exception as error:  # pylint: disable=broad-except
                raise RuntimeError("FZF error: {}".format(error))

        final_src = [os.path.join(select[0], *path_relative_home, fname)] + local_dist
        try:
            fzf = FzfPrompt()
            src = fzf.prompt(final_src)[0]
        except Exception as error:  # pylint: disable=broad-except
            raise RuntimeError("FZF error: {}".format(error))

        logger.info(f"{src} ====> {dst}")
        logger.info(f"ln -sfn {src} {dst}")
        if not is_debug:
            os.makedirs(src, exist_ok=True)
            os.symlink(src, dst)


def delete_link(
    fname: str,
    is_delete: bool = typer.Option(False, "--d", help="delete src folder"),
):
    """
    delete link
    """
    cwd = os.getcwd()
    dst = os.path.join(cwd, fname)
    if os.path.exists(dst):
        if os.path.islink(dst):
            src = os.readlink(dst)
            if is_delete:
                logger.warning(f" Deleting SRC: {src} ==> {dst}")
            else:
                logger.warning(f" Deleting link: {src} ==> {dst}")
            shutil.rmtree(dst, ignore_errors=True)
            if is_delete:
                shutil.rmtree(src, ignore_errors=True)
        else:
            logger.warning(f"{dst} is not a link")


def link():
    typer.run(create_link)


def unlink():
    typer.run(delete_link)



import datetime

from .utils import extract_time


def if_stale(run):
    """judge whether run is stale, runs happens long time ago or its duration is short

    :return: True if stale
    :rtype: bool
    """
    duration, run2now = extract_time(run)
    time_thred = [[0, 10], [7, 60], [3, 180], [1, 600]]
    for thr_d, thr_s in time_thred:
        if run2now > datetime.timedelta(days=thr_d) and duration < datetime.timedelta(
            seconds=thr_s
        ):
            return True
    return False


def if_notag(run):
    duration, run2now = extract_time(run)
    if "stable" in run.tags or "delete" in run.tags:
        return True
    if duration > datetime.timedelta(days=2):
        return False
    if run2now > datetime.timedelta(days=3) and len(run.tags) == 0:
        return True
    return False



import os

import typer
from pyfzf.pyfzf import FzfPrompt


def fzf_select(folder: str):  # pylint: disable=inconsistent-return-statements
    fzf = FzfPrompt()
    sub_f = os.listdir(folder)
    try:
        return fzf.prompt(sub_f, "--multi")
    except Exception as error:  # pylint: disable=broad-except
        print("FZF error: {}".format(error))


def star_folder(folder: str):
    select_f = fzf_select(folder)
    for cur_f in select_f:
        full_f = os.path.join(folder, cur_f)
        if cur_f[:3] == "j*_":
            continue
        target_f = os.path.join(folder, f"j*_{cur_f}")
        os.rename(full_f, target_f)


def unstar_folder(folder: str):
    select_f = fzf_select(folder)
    for cur_f in select_f:
        full_f = os.path.join(folder, cur_f)
        if cur_f[:3] == "j*_":
            target_f = os.path.join(folder, cur_f[3:])
            os.rename(full_f, target_f)


def star_main(
    folder: str = ".runs", unstar: bool = typer.Option(False, "-d", help="delete stars")
):
    if unstar:
        unstar_folder(folder)
    else:
        star_folder(folder)


def main():
    typer.run(star_main)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jamexp', 'jamexp.cli', 'jamexp.hyd', 'jamexp.utils', 'jamexp.wandb_clean']

package_data = \
{'': ['*']}

install_requires = \
['GitPython>=3.1.17,<4.0.0',
 'Pillow>=8.2.0,<9.0.0',
 'hydra-core>=1.0.6,<2.0.0',
 'loguru>=0.5.3,<0.6.0',
 'numpy>=1.20.3,<2.0.0',
 'pyfzf>=0.2.2,<0.3.0',
 'pylint>=2.8.3,<3.0.0',
 'scipy>=1.6.3,<2.0.0',
 'typer>=0.3.2,<0.4.0',
 'wandb>=0.10.30,<0.11.0']

entry_points = \
{'console_scripts': ['jhydc = jamexp.cli.jhyd_clean:main',
                     'jln = jamexp.cli.jlink:link',
                     'jpin = jamexp.cli.jpin:main',
                     'jstar = jamexp.cli.star_file:main',
                     'junln = jamexp.cli.jlink:unlink',
                     'jwdc = jamexp.cli.jwb_clean:main']}

setup_kwargs = {
    'name': 'jamexp',
    'version': '0.0.7',
    'description': 'Jam Experiment helper',
    'long_description': '# jamexp\n\n## cli\n\n* `jstar`: star files with fzf prompt.\n\n### `jwdc`: clear stale, short and notag\n```shell\nexport WANDB_ENTITY=name\nexport WANDB_API_KEY=key\n```\n\n* `stale`: [0, 10], [7, 60], [3, 180], [1, 600] (days, run_duration/seconds)\n* `short`: three days ago and no tags\n\n### `jhydc`: clear hyd exp folders\n\n* `stale`: [0, 10], [7, 60], [3, 180], [1, 600]\n* `wandb-sync`\n\n* `jpin`: pin good results\n\n### `jln` and `junln`\n',
    'author': 'Qinsheng',
    'author_email': 'qsh.zh27@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<3.10',
}


setup(**setup_kwargs)



import datetime
import os
import platform


def latest_time(fname):
    if platform.system() == "Windows":
        ftime = os.path.getctime(fname)
    else:
        stat = os.stat(fname)
        try:
            ftime = stat.st_birthtime
        except AttributeError:
            # probably on Linux.
            ftime = stat.st_mtime
    return datetime.datetime.fromtimestamp(ftime)



import typer

from jamexp.wandb_clean import delete_run, get_proj_runs, if_notag, if_stale


def clean_project(
    proj: str,
    name: str = typer.Option(None, "-n", help="run name"),
    is_delete: bool = typer.Option(False, "--d", help="delete runs"),
    is_all: bool = typer.Option(
        False, "--a", help="delete runs not run in this machine"
    ),
):
    """
    `stale`: [0, 10], [7, 60], [3, 180], [1, 600] (days, run_duration/seconds)
    `short`: three days ago and no tags
    """
    for run in get_proj_runs(proj):
        if if_stale(run) and if_notag(run):
            delete_run(run, is_delete, is_all)
        elif name is not None and name in run.name:
            delete_run(run, is_delete, is_all)


def main():
    typer.run(clean_project)



import os
import os.path as osp
import re
import shutil
from datetime import datetime, timedelta
from glob import glob

from loguru import logger
from omegaconf import OmegaConf

from jamexp.utils import latest_time
from jamexp.wandb_clean import get_run_from_url


def extract_all_exp(folder: str = "outputs"):
    """extract whole hyd runs under folder

    :param folder: [description], defaults to "outputs"
    :type folder: str, optional
    :return: list of pathes
    """
    rtn_exps = []
    if not osp.exists(folder):
        logger.error(f"{folder} does not exists")
        exit(1)
    for cur_path in glob(f"{folder}/*/*/"):
        # TODO: remove me. Should be a global variable or more customized!
        if re.search(r"\d{4}-\d{2}-\d{2}/\d{2}-\d{2}-\d{2}/$", cur_path):
            rtn_exps.append(cur_path)
    return sorted(rtn_exps, reverse=True)


def time_from_folder(folder_str: str):
    """hyd folder name 2 its creation time

    :param folder_str: [description]
    :type folder_str: str
    :return: datetime
    """
    assert folder_str[-1] == "/"
    exp_path_time = folder_str[-20:]
    return datetime.strptime(exp_path_time, "%Y-%m-%d/%H-%M-%S/")


def clear_empty(folder: str = "outputs"):
    """clear empty hyd runs folder

    :param folder: [description], defaults to "outputs"
    :type folder: str, optional
    """
    for cur_path in glob(f"{folder}/*/"):
        if len(os.listdir(cur_path)) == 0:
            shutil.rmtree(str(cur_path), ignore_errors=True)


def filter_hyd_stale(folder: str = "outputs"):
    to_delete = []
    for run_f in extract_all_exp(folder):
        create_time = time_from_folder(run_f)
        end_time = latest_time(run_f)
        now = datetime.now()
        run2now = now - end_time
        duration = end_time - create_time
        time_thred = [[0, 10], [7, 60], [3, 180], [1, 600]]
        for thr_d, thr_s in time_thred:
            if run2now > timedelta(days=thr_d) and duration < timedelta(seconds=thr_s):
                wandb_run = run_from_meta(run_f)
                if wandb_run is not None:
                    logger.info(f"SHORT: {run_f}\t {wandb_run.url}")
                else:
                    logger.info(f"SHORT: {run_f}")
                to_delete.append(run_f)
                break
    return to_delete


def run_from_meta(run_f: str):
    meta_f = osp.join(run_f, "meta.yaml")
    if osp.exists(meta_f):
        conf = OmegaConf.load(meta_f)
        if "WEIGHT_URL" in conf:
            run = get_run_from_url(conf["WEIGHT_URL"])
            if run is not None:
                return run
    return None


def delete_run_from_folder(run_f: str):
    run = run_from_meta(run_f)
    if run is not None:
        run.delete()


def filter_hyd_wandb(folder: str = "outputs"):
    to_delete = []
    for run_f in extract_all_exp(folder):
        if run_from_meta(run_f) is None:
            to_delete.append(run_f)
    return to_delete
